@@ -1,8 +1,7 @@
-from gql import gql, Client#, AIOHTTPTransport, RequestsHTTPTransport # This is gql version 3
+from gql import gql, Client
 from gql.transport.requests import RequestsHTTPTransport
 from loguru import logger
 import base64
-from numpy.core.records import array
 
 import pandas
 import sys
@@ -10,6 +9,7 @@ import json
 import os
 import requests
 import pytz
+from tzlocal import get_localzone
 import getpass
 
 from .fileimport import FileImport
@@ -17,10 +17,11 @@ from .fileImportService import FileImportService
 from .automation import Automation, Schedule
 from .programming import Programming
 
-from .utils.ut_core import Defaults, Utils
+from .utils.ut import Utils
 from .utils.ut_meta import Meta
 from .timeseries import TimeSeries
 
+
 class TechStack(Utils, Meta):
     """
     Initializes a Seven2one TechStack client.
@@ -31,16 +32,22 @@ class TechStack(Utils, Meta):
         The host domain or ip address, e.g. 'app.organisation.com'
     user: str
         A username
-    password: str=None
+    password: str = None
         The password belonging to the user. If None, it needs to
         be entered interactively.
-    logLevel: str='WARNING'
+    logLevel: str = 'WARNING'
         Change the highest log level. Available log levels are 
         'ERROR', 'WARNING', 'INFO' and 'DEBUG'.
-    usePorts: bool=False
+    usePorts: bool = False
         A developer feature: if True, ports with prefix '8' for a
         developer ComposeStack will be used.
-    copyGraphQLString: bool=False
+    timeZone: str = None
+        A time zone provided in IANA or isoformat (e.g. 'Europe/Berlin' or 'CET').
+        Defaults to the local timezone. 
+    dateTimeOffset: bool = True
+        Choose, if a Timestamp should be displayed as time zone naive (dateTimeOffset = False) 
+        or with Offset information. 
+    copyGraphQLString: bool = False
         A developer feature: if True, with each method execution, 
         the graphQL string is copied to the clipboard (Windows only).
 
@@ -49,7 +56,8 @@ class TechStack(Utils, Meta):
     """
 
     def __init__(self, host:str, user:str, password:str=None, logLevel:str='WARNING', 
-        usePorts:bool=False, copyGraphQLString:bool=False) -> object:
+        usePorts:bool=False, timeZone:str=None, dateTimeOffset:bool=True, 
+        copyGraphQLString:bool=False) -> object:
                
         try:
             logger.remove()
@@ -72,6 +80,7 @@ class TechStack(Utils, Meta):
             scheduleEndpoint = f'https://{host}/schedule/graphql/'
             programmingEndpoint = f'https://{host}/programming/graphql/'
             tsGatewayEndpoint = f'https://{host}/timeseries/graphql/'
+            schemeUrl = f'https://{host}/dynamic-objects/graphql?sdl'
             #fileImportServiceEndpoint = f'https://{host}/programming/graphql/'
         else: 
             tokenUrl = f'http://{host}:8040/connect/token'
@@ -81,6 +90,7 @@ class TechStack(Utils, Meta):
             programmingEndpoint = f'http://{host}:8140/graphql/'
             tsGatewayEndpoint = f'http://{host}:8195/graphql/'
             fileImportServiceEndpoint = f'http://{host}:7200/graphql/'
+            schemeUrl = f'https://{host}:8050/graphql?sdl'
 
         body = {
             'client_id': 'techstack-ui',
@@ -106,14 +116,11 @@ class TechStack(Utils, Meta):
         transport =  RequestsHTTPTransport(url=dynEndpoint, headers=header, verify=True)
         self.client = Client(transport=transport, fetch_schema_from_transport=True)  
         self.accessToken = token
+        if self.client.introspection == None:
+            self.client = Client(transport=transport, schema=schemeUrl, headers=header)  
         self.scheme = self.client.introspection
-
-        # Defaults:
-        Utils._getDefaults()
-        Defaults.copyGraphQLString = copyGraphQLString
-            
-        # self.timeZone = defaults['timeZone']
-        # self.dateTimeFormat = defaults['dateTimeFormat']
+        if self.scheme == None:
+            logger.error(f"Schema download failed.")
 
         try:
             self.TimeSeries = TimeSeries(self.accessToken, tsGatewayEndpoint, self)
@@ -147,6 +154,14 @@ class TechStack(Utils, Meta):
             logger.warning(f"Programming service not available")
             logger.debug(f"Reason: {err}")
 
+        # Defaults:
+        if timeZone == None:
+            Utils.timeZone = Utils._timeZone('local')
+        else: 
+            Utils.timeZone = timeZone
+        Utils.useDateTimeOffset = dateTimeOffset
+        Utils.copyGraphQLString = copyGraphQLString
+
         return
 
     def inventories(self, fields:list=None, where:str=None, orderBy:str=None, 
@@ -196,22 +211,21 @@ class TechStack(Utils, Meta):
 
         graphQLString= f'''query getInventories {{
         inventories 
-            (first: 50 {_orderBy} {resolvedFilter})
+            (pageSize:1000 {_orderBy} {resolvedFilter})
             {{
-            nodes {{
-                {_fields}
-                }}
+            {_fields}
             }}
         }}
         '''
         result = Utils._executeGraphQL(self, graphQLString)
         if result == None: return
 
-        df = pandas.json_normalize(result['inventories']['nodes'])
+        df = pandas.json_normalize(result['inventories'])
         return df
 
     def items(self, inventoryName:str, references:bool=False, fields:list=None, where:str=None, 
-        pageSize:int=5000, orderBy:str=None, asc:bool=True) -> pandas.DataFrame:
+        orderBy:str=None, asc:bool=True, pageSize:int=5000, 
+        arrayPropertiesToPage:list=None, arrayPageSize:int=50) -> pandas.DataFrame:
         """
         Returns items of an inventory in a DataFrame.
 
@@ -219,35 +233,41 @@ class TechStack(Utils, Meta):
         -----------
         inventoryName : str
             The name of the inventory.
-        references : bool
+        references: bool = False
             If True, items of referenced inventories will be added to the DataFrame. If
             the fields-parameter is used, this parameter is ignored.
-        fields : list | str
+        fields: list | str = None
             A list of all properties to be queried. If None, all properties will be queried.
             For referenced items use a '.' between inventory name and property.
-        pageSize : int
-            The page ize of items that is used to retrieve a large number of items.
-        where : str
+        where: str = None
             Use a string to add where criteria like
             'method eq "average" and location contains "Berlin"'.
             Referenced items are not supported.
-        orderBy : str
+        orderBy: str = None
             Select a field to sort by.
-        asc : bool
+        asc: bool = True
             Determines the sort order of items. If set to False, a descending order 
             is applied.
+        pageSize: int = 5000
+            The page size of items that is used to retrieve a large number of items.
+        arrayPropertiesToPage:list = None
+            Array (list) properties that should be paged. Real paging for array properties is 
+            not yet supported. The properties listed here will just make use of of the arrayPageSize.
+        arrayPageSize: int = 50
+            The page size of list items that is used to page list items in an inventory item.
 
         Example:
         --------
         >>> items('appartments', references=True)
         """
 
-
-        # tz = globalTimeZone
-
+        deleteId = False
         if fields != None:
             if type(fields) != list:
                 fields = [fields]
+            if 'sys_inventoryItemId' not in fields:
+                deleteId = True
+                fields += ['sys_inventoryItemId']
             _fields = Utils._queryFields(fields, recursive=True)
         else:
             properties = Utils._properties(self, inventoryName, recursive=True)
@@ -270,21 +290,28 @@ class TechStack(Utils, Meta):
                 _orderBy = f'order: {{ {orderBy}: ASC }}'
         else: _orderBy = ''
 
+        if arrayPropertiesToPage != None:
+            listPagination = 'listPagination: [\n'
+            for arrayProperty in arrayPropertiesToPage:
+                listPagination += f'{{ from: 0, size: {arrayPageSize}, propertyPath: "{arrayProperty}" }}\n'
+            listPagination += ']\n'
+        else:
+            listPagination = ''
+
         result = []
         count = 0
         lastId = ''
 
         while True:
             graphQLString= f''' query getItems {{
-                    {inventoryName} 
-                    (pageSize: {pageSize} {lastId} {resolvedFilter} {_orderBy})
-                    {{
-                        edges {{
-                            cursor
-                            node {{
-                                {_fields}
-                            }}
-                        }}
+                    {inventoryName} (
+                            pageSize: {pageSize} 
+                            {listPagination}
+                            {lastId}
+                            {resolvedFilter}
+                            {_orderBy}
+                            ) {{
+                        {_fields}
                     }}
                 }}
                 '''
@@ -303,24 +330,24 @@ class TechStack(Utils, Meta):
                 logger.error(err)
                 return
             
-            if _result[inventoryName]['edges']:
-                result += _result[inventoryName]['edges']
+            if _result[inventoryName]:
+                result += _result[inventoryName]
                 count += 1
             try:
-                cursor = _result[inventoryName]['edges'][-1]['node']['_inventoryItemId']
+                cursor = _result[inventoryName][-1]['sys_inventoryItemId']
                 lastId = f'lastId: "{cursor}"'
             except: 
                 break
 
         df = pandas.json_normalize(result)
-        # Remove cursor columns and remove 'node' prefix
-        try:
-            del df['cursor']
-        except: pass
-
-        cols = [col.replace('node.','') for col in df.columns]
-        df.columns = cols
         
+        if fields != None: #sorts dataframe according to given fields
+            try: # for array field this does not work
+                df = df[fields]
+            except:
+                pass 
+        if deleteId: del df['sys_inventoryItemId']
+
         return df
 
     def inventoryProperties(self, inventoryName, namesOnly=False):
@@ -342,13 +369,13 @@ class TechStack(Utils, Meta):
         """
 
         propertyFields = f'''
-            name
-            id
-            type
+            name          
             ... Scalar
             isArray
             nullable
             ... Reference
+            type
+            propertyId
         '''
 
         graphQLString= f'''query Inventory {{
@@ -399,13 +426,10 @@ class TechStack(Utils, Meta):
 
         """
 
-        if references != True:
-            _properties = Utils._properties(self, inventoryName=inventoryName)
-        else:
-            _properties = Utils._properties(self, inventoryName=inventoryName, recursive=True)
+        _properties = Utils._properties(self, inventoryName=inventoryName, recursive=references)
 
         if dataTypes == False:
-            properties = Utils._propertyList(_properties)
+            properties = Utils._propertyList(_properties, recursive=references)
         else:
             properties = pandas.Series(Utils._propertyTypes(_properties))
 
@@ -438,18 +462,16 @@ class TechStack(Utils, Meta):
         >>> addItems('meterData', items)
         """
         items = Utils._propertiesToString(items)
-        #_inventoryFieldName = utils._upperFirst(inventoryFieldName)
+        key = f'create{inventoryName}'
 
         graphQLString= f'''mutation addItems {{
-            create{inventoryName} (input: 
+            {key} (input: 
                 {items}
             )
-            {{
-                errors {{
-                    message
-                }}
-                    InventoryItems {{
-                _inventoryItemId
+                {{
+                    {Utils.errors}           
+                inventoryItems {{
+                    sys_inventoryItemId
                 }}
             }}
         }} 
@@ -457,15 +479,13 @@ class TechStack(Utils, Meta):
         result = Utils._executeGraphQL(self, graphQLString)
         if result == None: return
 
-        key = f'create{inventoryName}'
-
         if result[key]['errors']:
             Utils._listGraphQlErrors(result, key)
-        return result[key]['InventoryItems']
+        return result[key]['inventoryItems']
 
     def updateItems(self, inventoryName:str, items:list) -> None:
         """
-        Updates from a list of dicts existing items. The '_inventoryItemId'
+        Updates from a list of dicts existing items. The 'sys_inventoryItemId'
         must be passed to each item.
 
         Parameters:
@@ -479,12 +499,12 @@ class TechStack(Utils, Meta):
         --------
         >>> items = [
                 {
-                '_inventoryItemId':'118312438662692864',
+                'sys_inventoryItemId':'118312438662692864',
                 'meterId': '86IEDD99',
                 'dateTime': '2020-01-01T05:50:59Z'
                 },
                 {
-                '_inventoryItemId':'118312438662692864',
+                'sys_inventoryItemId':'118312438662692864',
                 'meterId': '45IXZ52',
                 'dateTime': '2020-01-07T15:41:14Z'
                 }
@@ -492,31 +512,28 @@ class TechStack(Utils, Meta):
         >>> updateItems('meterData', items)
         """
         items = Utils._propertiesToString(items)
-        #_inventoryFieldName = utils._upperFirst(inventoryFieldName)
+        #items = items.replace('"null"', 'null')
+        key = f'update{inventoryName}'
 
         graphQLString= f'''mutation updateItems {{
-            update{inventoryName} (input: 
+            {key} (input: 
                 {items}
             )
-            {{
-                errors {{
-                    message
-                }}
-                InventoryItems {{
-                    _inventoryItemId
+                {{
+                    {Utils.errors}           
+                    inventoryItems {{
+                        sys_inventoryItemId
                 }}
             }}
-        }} 
+        }}
         '''
         result = Utils._executeGraphQL(self, graphQLString)
         if result == None: return
 
-        key = f'update{inventoryName}'
-
         if result[key]['errors']:
             Utils._listGraphQlErrors(result, key)
 
-        logger.info(f"{len(result[key]['InventoryItems'])} item(s) updated.")
+        logger.info(f"{len(result[key]['inventoryItems'])} item(s) updated.")
         return
 
     def createInventory(self, name:str,  properties:list, variant:str=None, 
@@ -570,12 +587,13 @@ class TechStack(Utils, Meta):
         _properties = Utils._propertiesToString(properties)
 
         if variant != None:
-            _variantId = Utils._getVariantId(self.variants(), variant) 
+            _variantId = f'{Utils._getVariantId(self.variants(), variant)}'
             logger.debug(f"Found variantId: {_variantId}")
             if type(_variantId) != str:
-                logger.error(f"Variant name '{name} not found")
+                logger.error(f"Variant name '{name}' not found")
                 return
-        else: _variantId = ''
+            _variant = f'variantId: "{_variantId}"'
+        else: _variant = ''
 
         if propertyUniqueness != None:
             _propertyUniqueness = 'propertyUniqueness: ' + Utils._uniquenessToString(propertyUniqueness)
@@ -586,27 +604,22 @@ class TechStack(Utils, Meta):
         _validityPeriods = 'true' if hasValitityPeriods != False else 'false'
         _domainUser = 'true' if isDomainUserType != False else 'false'
         
-
-        
         graphQLString= f'''
         mutation createInventory {{
             createInventory (input: {{
                 name: "{name}"        
                 properties: {_properties}
-                variantId: "{_variantId}"
+                {_variant}
                 historyEnabled: {_history}
                 hasValidityPeriods: {_validityPeriods}
                 isDomainUserType: {_domainUser}
                 {_propertyUniqueness}
                 }})
-            {{
-            inventory {{
-                inventoryId
-            }}
-            errors {{
-                code
-                message
-                }}
+                {{
+                inventory {{
+                    inventoryId
+                    }}
+                {Utils.errors}           
             }}
         }}
         '''
@@ -641,29 +654,29 @@ class TechStack(Utils, Meta):
         
         if inventory.empty:
             logger.error(f'Unknown inventory "{inventoryName}".')
+            return
         inventoryId = inventory.loc[0, 'inventoryId']
-        logger.debug(f'_inventoryId: {inventoryId}')
+        logger.debug(f'inventoryId: {inventoryId}')
 
         if force == False:
             confirm = input(f"Press 'y' to delete '{inventoryName}': ")
 
+        key = 'deleteInventory'
         graphQLString= f'''mutation deleteInventory {{
-            deleteInventory (input:{{inventoryId: "{inventoryId}"}})
-            {{ 
-                errors {{ 
-                    code
-                    message
+            {key} (input:{{inventoryId: "{inventoryId}"}})
+                {{
+                    {Utils.errors}           
                 }}
             }}
-            }}
             '''
 
         if force == True: confirm = 'y'
         if confirm == 'y':
             result = Utils._executeGraphQL(self, graphQLString)
             if result == None: return
-            if result['deleteInventory']['errors']:
-                Utils._listGraphQlErrors(result, 'deleteInventory')
+            if result[key]['errors']:
+                Utils._listGraphQlErrors(result, key)
+                return
             else: 
                 logger.info(f"Inventory '{inventoryName}' deleted.")
         else: return
@@ -673,21 +686,18 @@ class TechStack(Utils, Meta):
             Returns a dataframe of available variants.
         """
 
+        key = 'variants'
         graphQLString= f'''query getVariants {{
-        variants
-            (first: 50)
-            {{
-            nodes {{
-                name
-                variantId
-                }}
+        {key} {{
+            name
+            variantId
             }}
         }}
         '''
         result = Utils._executeGraphQL(self, graphQLString)
         if result == None: return
 
-        df = pandas.json_normalize(result['variants']['nodes'])
+        df = pandas.json_normalize(result[key])
         return df
 
     def clearInventory(self, inventoryName:str, force=False):
@@ -702,31 +712,29 @@ class TechStack(Utils, Meta):
             Use True to ignore confirmation.
         """
 
-        _result =self.items(inventoryName, ['_inventoryItemId', 'inventoryId'])
+        _result = self.items(inventoryName, ['sys_inventoryItemId', 'inventoryId'])
         if _result.empty:
-            logger.info(f"Inventory {inventoryName} has no items.")
+            logger.info(f"The inventory is empty.")
             return
-        _inventoryItemIds = list(_result['_inventoryItemId'])
+        _inventoryItemIds = list(_result['sys_inventoryItemId'])
 
         _ids = ''
         for id in _inventoryItemIds:
-            _ids += f'{{_inventoryItemId: "{id}"}}\n'    
+            _ids += f'{{sys_inventoryItemId: "{id}"}}\n'    
       
         logger.debug(f"GraphQL Ids: {_ids}")
             
         if force == False:
             confirm = input(f"Press 'y' to delete  {len(_inventoryItemIds)} items: ")
 
+        key = f'delete{inventoryName}'
         graphQLString= f'''
             mutation deleteItems {{
-                delete{inventoryName} ( input: 
+                {key} ( input: 
                     [{_ids}]
                 )
                 {{
-                errors {{
-                    code
-                    message
-                    }}
+                    {Utils.errors}           
                 }}
             }}
             '''
@@ -738,8 +746,8 @@ class TechStack(Utils, Meta):
         else:
             return
 
-        if result[f'delete{inventoryName}']['errors'] != None:
-            Utils._listGraphQlErrors(result, f'delete{inventoryName}')
+        if result[key]['errors'] != None:
+            Utils._listGraphQlErrors(result, key)
         else:
             logger.info(f"{len(_inventoryItemIds)} items deleted.")
 
@@ -773,36 +781,34 @@ class TechStack(Utils, Meta):
             logger.warning(f"List of items and where-criteria has been provided. Item list is used.")
 
         if where != None:
-            _result =self.items(inventoryName, ['inventoryItemId', 'inventoryId'], 
+            _result =self.items(inventoryName, ['sys_inventoryItemId', 'inventoryId'], 
                 where=where)
             if _result.empty:
                 logger.info(f"The where criteria '{where}' led to no results.")
                 return
 
-            _inventoryItemIds = list(_result['_inventoryItemId'])
+            _inventoryItemIds = list(_result['sys_inventoryItemId'])
             
         if inventoryItemIds != None:
             _inventoryItemIds = inventoryItemIds
 
         _ids = ''
         for id in _inventoryItemIds:
-            _ids += f'{{_inventoryItemId: "{id}"}}\n'    
+            _ids += f'{{sys_inventoryItemId: "{id}"}}\n'    
       
         logger.debug(f"GraphQL Ids: {_ids}")
             
         if force == False:
             confirm = input(f"Press 'y' to delete  {len(_inventoryItemIds)} items: ")
 
+        key = f'delete{inventoryName}'
         graphQLString= f'''
             mutation deleteItems {{
-                delete{inventoryName} ( input: 
+                {key} ( input: 
                     [{_ids}]
                 )
                 {{
-                errors {{
-                    code
-                    message
-                    }}
+                    {Utils.errors}           
                 }}
             }}
             '''
@@ -815,8 +821,8 @@ class TechStack(Utils, Meta):
         else:
             return
 
-        if result[f'delete{inventoryName}']['errors'] != None:
-            Utils._listGraphQlErrors(result, f'delete{inventoryName}')
+        if result[key]['errors'] != None:
+            Utils._listGraphQlErrors(result, key)
         else:
             logger.info(f"{len(_inventoryItemIds)} items deleted.")
 
@@ -825,41 +831,32 @@ class TechStack(Utils, Meta):
 
         _variantId = Utils._getVariantId(self.variants(), variantName) 
         logger.debug(f"Found variantId: {_variantId}")
-        if type(_variantId) != str:
-            logger.er
 
         if newName != None:
             _name = f'name: "{newName}"'
         else:
-            _name = ''
+            _name = 'null'
 
         if icon != None:
             _icon =f'icon: "{icon}"'
         else:
-            _icon =''
+            _icon = 'null'
 
+        key = 'updateVariant'
         graphQLString = f'''mutation updateVariant {{
-            updateVariant (input:{{
+            {key} (input:{{
                 variantId: "{_variantId}"
                 {_name}
                 {_icon}
                 }})
-            {{ 
-                errors {{ 
-                    code
-                    message
+                {{
+                    {Utils.errors}           
                 }}
             }}
-            }}
             '''
 
         result = Utils._executeGraphQL(self, graphQLString)
         if result == None: return
-        print(result)
-        # if result['updateVariant']['errors'] != 'None':
-        #     Utils._listGraphQlErrors(result, 'updateVariant')
-        # else: 
-        #     logger.info(f"Variant '{variantName}' updated.")
 
         return
 
@@ -874,13 +871,13 @@ class TechStack(Utils, Meta):
         inventoryName : str
             The name of the inventory where the item is located.
         inventoryItemId : str
-            The inventory item id of the item.
+            The sys_inventoryItemId of the item.
         arrayProperty : str
             The name of the property whose array items are to be updated.
         operation : str
             The update operation to be performed. Options:
             insert: inserts a list of array items.
-            removeById: removes array items by a list of given (item) ids.
+            removeById: removes array items by a list of given ids.
             removeByIndex: removes array items by a list of given indices.
             removeAll: removes all array items
         cascadeDelete : bool (False)
@@ -888,47 +885,55 @@ class TechStack(Utils, Meta):
 
         Examples:
         ---------
-        >>> client.updateArrayProperty('meterData', '118718694909018112',
-                'timeSeries', action='insert', arrayItems=['387964598252380179'])
-        >>> client.updateArrayProperty('meterData', '118718694909018112',
+        >>> client.updateArrayProperty('meterData', 'A5N45hOOmm',
+                'timeSeries', action='insert', arrayItems=['A5FdSjehbE'])
+        >>> client.updateArrayProperty('meterData', 'A5N45hOOmm',
                 'timeSeries', action='removeByIndex', arrayItems=[0,1], cascadeDelete=True)
         """
 
-
         operations = ['insert', 'removeById', 'removeByIndex', 'removeAll']
         if operation not in operations:
             logger.error(f"Action '{operation}' allowed. Possible update operations: {operations} ")
 
         if operation == 'removeAll':
             try:
-                arrDf= TechStack.items(self, inventoryName, fields=[f'{arrayProperty}._inventoryItemId'],
-                    where=f'_inventoryItemId eq "{inventoryItemId}"')
+                propertyDf = TechStack.inventoryProperties(self, 'volueGroups')
+                arrayProps = propertyDf[propertyDf['name'] == arrayProperty]
+                logger.debug(f"Properties of array: {arrayProps}")
+                if arrayProps['type'].item() == 'reference':
+                    arrDf = TechStack.items(self, inventoryName, 
+                        where=f'sys_inventoryItemId eq "{inventoryItemId}"', fields=[f'{arrayProperty}.sys_inventoryItemId'],
+                        arrayPropertiesToPage=[arrayProperty], arrayPageSize=5000)
+                else:
+                    arrDf = TechStack.items(self, inventoryName, fields=[{arrayProperty}],
+                        where=f'sys_inventoryItemId eq "{inventoryItemId}"', listPageSize=5000)
             except Exception as err:
                 logger.error(err)
                 return
-            countArray = len(arrDf.iloc[0,0])
+            countArray = len(arrDf[arrayProperty].item())
             arrayItems = [num for num in range(countArray)]
+            logger.debug(f"Array Items: {arrayItems}")
         _arrayItems = Utils._arrayItemsToString(arrayItems, operation, cascadeDelete)
+        logger.debug(f"Array Items as String: {_arrayItems}")
 
+        key = f'update{inventoryName}ArrayProperties'
         graphQLString = f'''mutation updateArray {{
-            update{inventoryName}ArrayProperties(
+            {key} (
                 input: {{
-                _inventoryItemId: "{inventoryItemId}"
+                sys_inventoryItemId: "{inventoryItemId}"
                 {arrayProperty}: {{
                     {_arrayItems}
                     }}
                 }}
-            ) {{
-                errors {{
-                message
-                }}
+            ) 
+            {{
+                {Utils.errors}           
             }}
         }}'''
 
         result = Utils._executeGraphQL(self, graphQLString)
         if result == None: return
 
-        key = f'update{inventoryName}ArrayProperties'
         if result[key]['errors'] != None:
             Utils._listGraphQlErrors(result, key)
         else:
@@ -969,9 +974,7 @@ class TechStack(Utils, Meta):
             properties: {_properties}
             }}) 
             {{
-                errors {{
-                    message
-                }}
+                {Utils.errors}           
             }}
         }}
         '''
@@ -984,6 +987,35 @@ class TechStack(Utils, Meta):
         else:
             logger.info(f"New property(ies) added.")
 
+    def updateDisplayValue(self, inventoryName, displayValue) -> None:
+
+        inventory = self.inventories(where=f'name eq "{inventoryName}"')
+        
+        if inventory.empty:
+            logger.error(f'Unknown inventory "{inventoryName}".')
+            return
+        inventoryId = inventory.loc[0, 'inventoryId']
+        logger.debug(f'inventoryId: {inventoryId}')
+
+        key = 'updateInventory'
+        graphQLString= f'''mutation updateDisplayValue {{
+            {key} (input: {{
+                inventoryId: "{inventoryId}"
+                displayValue: "{displayValue}"
+                }}) {{
+                {Utils.errors}
+            }}
+        }}
+        '''
+        result = Utils._executeGraphQL(self, graphQLString)
+        if result == None: return
+
+        if result[key]['errors']:
+            Utils._listGraphQlErrors(result, key)
+            return
+        else:
+            logger.info(f"Display value updated.")
+
     def resync(self) -> None:
         """Resynchornizes read databases"""
 
@@ -1002,86 +1034,24 @@ class TechStack(Utils, Meta):
         pandas.options.display.max_rows = maxRows
         pandas.options.display.max_columns = maxColumns
         return
+
+    def defaults(self):
+        """Shows default values"""
+        defaults = {
+            'Time zone': Utils.timeZone,
+            'Use DateTimeOffset format': Utils.useDateTimeOffset,
+            'Copy graphQL string': Utils.copyGraphQLString
+            }
+
+        return defaults
+
 # Some stand alone functions
 
-def setGlobalDefault(default:str, value:object, verbose:bool=False) -> None:
-    """
-    Sets a default value for a specific option.
-    Available defaults:
-    timeZone : str
-        A time zone provided in IANA or isoformat (e.g. 'Europe/Berlin' or 'CET')
-    dateTimeFormat: str
-        Use 'dateTimeOffSet' or 'dateTime'
-
-    Parameters:
-    -----------
-    default : str
-        Choose a default, e.g. 'timeZone'.
-    value : object
-        Choose a value to be set as default.
-    verbose : bool
-        If True, config file information will be shown.
-    """
-    #global globalTimeZone, globalDateTimeFormat
-
-    ## Check for valid defaults:
-    defaults = ['timeZone', 'dateTimeFormat', 'copyGraphQLString']
-    if default not in defaults:
-        raise ValueError(f"Unknown default '{default}'.")
-
-    ## Check config file and create if not existing:
-    path = os.path.abspath(__file__)
-    path = path.replace('core.py', 'config.json')
-    if verbose: print(f"Path to config file: '{path}'")
-
-    try:
-        with open(path, 'r') as configFile:
-            content = json.load(configFile)
-    except:
-        print('I excepted')
-        with open(path, 'w') as configFile:
-            content = {
-                'timeZone': 'local',
-                'dateTimeFormat': 'dateTimeOffset',
-                'copyGraphQLString': False
-                }
-            json.dump(content, configFile, indent=4)
-    
-    ## Check for valid values:
-    if default == 'timeZone':
-        pytz.timezone(value)
-        globalTimeZone = value
-    if default == 'dateTimeFormat':
-        if value not in ['dateTimeOffset', 'dateTime']:
-            raise ValueError(f"'{value}' is no valid DateTime format. Use 'dateTimeOffset' or 'dateTime'")
-        globalDateTimeFormat = value
-    if default == 'copyGraphQLString':
-        if value not in [True, False]:
-            raise TypeError(f"'{value}' is not valid. Use True or False.")
-    # if default == 'saveAccessToken':
-    #     if value not in [True, False]:
-    #         raise TypeError(f"'{value}' is not valid. Use True or False.")
-    # if default == 'tokenValidityHours':
-    #     if type(value) != int:
-    #         raise TypeError(f"'{value}' is not valid. Use an integer number.")
-
-    ## New content
-    content[default] = value
-    print(content)
-
-    ## Write to config file:
-    with open(path, 'w') as configFile:
-        #content = json.load(configFile)
-        if verbose: print(f"Current settings of config file: \n {content}")
-        json.dump(content, configFile, indent=4)
-    if verbose: print(f"{default} set to {value}.")
-    return
-
 def encodeBase64(file:str):
     with open(file) as file:
         content = file.read()
         content = base64.b64encode(content.encode('ascii'))
         return content.decode('UTF8')
         
-def decodeBase43(content:str):
+def decodeBase64(content:str):
     return base64.b64decode(content)
\ No newline at end of file
