@@ -1,7 +1,8 @@
 import csv
 import copy
 from pathlib import Path
-from datetime import datetime
+from pprint import pprint
+from datetime import timedelta
 
 from loguru import logger
 
@@ -212,7 +213,7 @@ class FileImport():
     def importTimeSeriesData(self, filePath:str, inventoryName:str=None, 
         importKeyProperty:str=None, delimiter:str=',', timeZone:str=None, 
         dateTimeFormat:str=None, fromTimepoint:str=None, toTimepoint:str=None, 
-        chunkSize:int=20000) -> None: 
+        timeDelta:timedelta=None, chunkSize:int=20000) -> None: 
         """
         Imports time series data from a specific CSV file or a folder with multiple
         CSV files. The first column is the timestamp index, whereas the first row
@@ -224,9 +225,7 @@ class FileImport():
         -----------
         filePath: str
             A path to a folder or a specific CSV file
-        delimiter: str
-            The CSV delimiter. Choose ',', ';', or 'tab'.
-        inventoryName: str = None
+        inventoryName: str
             The field name of the inventory, if not provided in the import file. 
             In the import file the inventory nameis located at position (0,0).
         importKeyProperty: str = None
@@ -234,7 +233,7 @@ class FileImport():
             values with time series. As an alternative, the content of a specific
             property can be used as header which will be mapped with the time series.
             This property should be unique.
-        delimiter : str = ','
+        delimiter: str = ','
             The CSV delimiter. Choose ',', ';', or 'tab'.
         timeZone: str = None
             A time zone provided in IANA or isoformat (e.g. 'Europe/Berlin' or 'CET'). Defaults
@@ -243,10 +242,29 @@ class FileImport():
             Several date-time formats are supported, however, a custom format according to
             datetime.strftime() and strptime() format codes can be passed to convert the
             timestamp.
-        fromTimepoint : str = None
+        fromTimepoint: str = None
             Specify a timestamp in isoformat from which data should be imported.
-        toTimepoint : str = None
+        toTimepoint: str = None
             Specify a timestamp in isoformat until which data should be imported.
+        timeDelta: datetime.timedelta = None
+            Define a time delta to add or substract to the original timestamp.
+        chunkSize: int = 20000
+            Determines the number of time series datapoints which are written per chunk. Using chunks
+            can be necessary to avoid overloading.
+
+        Example:
+        --------
+        >>> client.FileImport.importTimeSeriesData(
+            filePath=file,
+            inventoryName='meterData'
+            importKeyProperty='name'
+            delimiter=';', 
+            timeZone='CET', 
+            dateTimeFormat='%Y-%b-%d %H:%M:%S'
+            fromTimePoint='2023-01-01'
+            toTimepoint='2023-03-05'
+            timeDelta=-timedelta(hours=1))
+            chunkSize=20000
         """
 
         ## TIMEZONE
@@ -259,7 +277,6 @@ class FileImport():
                 logger.error(f"File path {filePath} does not exist")
                 return
             files = [filePath]
-
         else:
             if not filePath.exists():
                 logger.error(f"File path {filePath} does not exist")
@@ -302,7 +319,8 @@ class FileImport():
 
             for i, row in enumerate(content):
                 try:
-                    content[i][0] = FileUtils._convertTimestamp(content[i][0], timeZone, dateTimeFormat)
+                    if content[i][0] == '': continue
+                    content[i][0] = FileUtils._convertTimestamp(content[i][0], timeZone, dateTimeFormat, timeDelta)
                 except Exception as err: 
                     if i >= 1:
                         logger.error(f"Timestamp {row[0]} could not be converted. {err}")
@@ -312,8 +330,7 @@ class FileImport():
             ## GET ITEM ID FROM IMPORT KEY
             if importKeyProperty != None:
                 try:
-                    items = core.TechStack.items(coreClient, inventoryName, fields=['unit', 'resolution', '_inventoryItemId', importKeyProperty])
-                    
+                    items = core.TechStack.items(coreClient, inventoryName, fields=['unit', 'resolution', 'sys_inventoryItemId', importKeyProperty])
                 except Exception as err:
                     logger.error(err)
                     return
@@ -322,7 +339,7 @@ class FileImport():
                 items = items[items[importKeyProperty].isin(names)]
                 idMapping = {}
                 for item in items.iterrows():
-                    idMapping.setdefault(item[1][importKeyProperty],item[1]['_inventoryItemId'])
+                    idMapping.setdefault(item[1][importKeyProperty],item[1]['sys_inventoryItemId'])
 
                 if len(idMapping) == 0:
                     logger.error(f"No item ids for importKeyProperty '{importKeyProperty}' found.")
@@ -331,7 +348,7 @@ class FileImport():
                 logger.debug(f"Id Mapping: {idMapping}")
             else:
                 try:
-                    items = core.TechStack.items(coreClient, inventoryName, fields=['unit', 'resolution', '_inventoryItemId'])
+                    items = core.TechStack.items(coreClient, inventoryName, fields=['unit', 'resolution', 'sys_inventoryItemId'])
                     logger.debug("Inventory read for default option (import with ivnentory item ids).")
                 except Exception as err:
                     logger.error(err)
@@ -369,7 +386,7 @@ class FileImport():
                         continue                
 
                 try:
-                    properties = items[items['_inventoryItemId'] == inventoryItemId]
+                    properties = items[items['sys_inventoryItemId'] == inventoryItemId]
                     if properties.empty:
                         logger.warning(f"Inventory item id {inventoryItemId} not found.")
                         output[fileName]['Time series errors'] += 1
@@ -383,23 +400,25 @@ class FileImport():
                 timeUnit = properties['resolution'].split(' ')[-1]
                 factor = properties['resolution'].split(' ')[0]
 
+                tsDict = {
+                        'sys_inventoryId': inventoryId,
+                        'sys_inventoryItemId': None,
+                        'data': {
+                            'resolution': {
+                                'timeUnit': None,
+                                'factor': None
+                            },
+                            'unit': None,
+                            'dataPoints': None
+                        }
+                    }
+
                 valueList = []
                 for i, row in enumerate(content):
-                    if i >= 1:
-                        tsDict = {
-                            '_inventoryId': inventoryId,
-                            '_inventoryItemId': None,
-                            'data': {
-                                'resolution': {
-                                    'timeUnit': None,
-                                    'factor': None
-                                },
-                                'unit': None,
-                                'dataPoints': None
-                            }
-                        }
-                    
+                    if i >= 1:     
+               
                         try:
+                            if row[0] == '': continue
                             if fromTimepoint:
                                 if row[0] < fromTimepoint: continue
                             if toTimepoint:
@@ -411,7 +430,9 @@ class FileImport():
                             errorDict[content[0][column]].setdefault(row[0], row[column])
                             output[fileName]['Value errors'] += 1     
 
-                tsDict['_inventoryItemId'] = inventoryItemId
+                logger.debug(f"Value list first 5 items: {valueList[:5]}")
+
+                tsDict['sys_inventoryItemId'] = inventoryItemId
                 tsDict['data']['unit'] = properties['unit']
                 tsDict['data']['resolution']['timeUnit'] = timeUnit
                 tsDict['data']['resolution']['factor'] = factor  
@@ -431,13 +452,10 @@ class FileImport():
                 tsChunk = copy.deepcopy(tsItemsEmpty)
                 for j, ts in enumerate(tsChunk):
                     tsChunk[j]['data'].setdefault('dataPoints', tsItems[j]['data']['dataPoints'][i : i + chunkSize])
-
-                timeseries.TimeSeries.setTimeSeriesDataCollection(self=tsClient, timeSeriesData=tsChunk)   
+                    
+                timeseries.TimeSeries.setTimeSeriesDataCollection(self=tsClient, timeSeriesData=tsChunk)    
                 logger.info(f"({int(i/chunkSize+1)}/{tsLength//chunkSize+1}) chunks imported.")            
 
-            # if len(errorDict) > 0:
-            #     output[fileName]['Errors'] = errorDict
-
         logger.info(f"Import finished")
 
         return 
@@ -484,7 +502,7 @@ class FileImport():
 
         if importKeyProperty != None:
             try:
-                items = core.TechStack.items(coreClient, groupInventoryName, fields=['_inventoryItemId', importKeyProperty])
+                items = core.TechStack.items(coreClient, groupInventoryName, fields=['sys_inventoryItemId', importKeyProperty])
             except Exception as err:
                 logger.error(err)
                 return
@@ -496,7 +514,7 @@ class FileImport():
 
             idMapping = {}
             for item in items.iterrows():
-                idMapping.setdefault(item[1][importKeyProperty],item[1]['_inventoryItemId'])
+                idMapping.setdefault(item[1][importKeyProperty],item[1]['sys_inventoryItemId'])
             logger.debug(f"Id mapping: {idMapping}")
             if len(idMapping) == 0:
                 logger.error(f"No item ids for importKeyProperty '{importKeyProperty}' found.")
@@ -527,7 +545,7 @@ class FileImport():
     def importGroupInstanceItemsWithData(self, filePath:str, groupInventoryName:str,
         instanceInventoryName:str, groupKeyProperty:str=None, instanceKeyProperties:list=None,
         delimiter:str=',', timeZone:str=None, dateTimeFormat:str=None, fromTimepoint:str=None, 
-        toTimepoint:str=None, chunkSize=20000, pause:int = 1) -> None:
+        toTimepoint:str=None, timeDelta:timedelta=None, chunkSize=20000, pause:int = 1) -> None:
         """
         Imports  group instance items from a CSV file. The CSV file only needs a header
         of property definitions. The first column is reserved for the groupInventoryItemId.
@@ -560,7 +578,12 @@ class FileImport():
             Specify a timestamp in isoformat from which data should be imported.
         toTimepoint : str = None
             Specify a timestamp in isoformat until which data should be imported.
-        pause : int
+        timeDelta: datetime.timedelta = None
+            Define a time delta to add or substract to the original timestamp.
+        chunkSize: int = 20000
+            Determines the number of time series datapoints which are written per chunk. Using chunks
+            can be necessary to avoid overloading.
+        pause: int = 1
             Between each chunk upload a pause is inserted in order to avoid overloading.
         """
 
@@ -627,7 +650,8 @@ class FileImport():
 
             for i, row in enumerate(content[dataBegin:]):
                 try:
-                    content[dataBegin+i][0] = FileUtils._convertTimestamp(content[dataBegin+i][0], timeZone, dateTimeFormat)
+                    if row[0] == '': continue
+                    content[dataBegin+i][0] = FileUtils._convertTimestamp(content[dataBegin+i][0], timeZone, dateTimeFormat, timeDelta)
                 except Exception as err: 
                     if i >= 1:
                         logger.error(f"Timestamp {row[0]} could not be converted. {err}")
@@ -646,7 +670,7 @@ class FileImport():
             ## PREPARE IMPORT: Get Mapping of Import Key
             if groupKeyProperty != None:
                 try:
-                    items = core.TechStack.items(coreClient, groupInventoryName, fields=['_inventoryItemId', groupKeyProperty])
+                    items = core.TechStack.items(coreClient, groupInventoryName, fields=['sys_inventoryItemId', groupKeyProperty])
                 except Exception as err:
                     logger.error(err)
                     return
@@ -658,7 +682,7 @@ class FileImport():
 
                 idMapping = {}
                 for item in items.iterrows():
-                    idMapping.setdefault(item[1][groupKeyProperty],item[1]['_inventoryItemId'])
+                    idMapping.setdefault(item[1][groupKeyProperty],item[1]['sys_inventoryItemId'])
                 logger.debug(f"Id mapping: {idMapping}")
                 if len(idMapping) == 0:
                     logger.error(f"No item ids for groupKeyProperty '{groupKeyProperty}' found.")
@@ -716,7 +740,7 @@ class FileImport():
                         x1 = content[instancePropPos[1]][i+1]
                         instanceTs = core.TechStack.items(coreClient, instanceInventoryName, 
                             where=f'{instanceKeyProperties[0]} eq "{x0}" and {instanceKeyProperties[1]} eq "{x1}"')
-                    inventoryItemId = instanceTs.loc[0, '_inventoryItemId']
+                    inventoryItemId = instanceTs.loc[0, 'sys_inventoryItemId']
                 except:
                     logger.error(f"Instance item in column {i+1} not found")
                     output[fileName]['Time series instance errors'] += 1
@@ -724,8 +748,8 @@ class FileImport():
                 sleep(pause)
 
                 tsDict = {
-                    '_inventoryId': inventoryId,
-                    '_inventoryItemId': inventoryItemId ,
+                    'sys_inventoryId': inventoryId,
+                    'sys_inventoryItemId': inventoryItemId ,
                     'data': {
                         'resolution': {
                             'timeUnit': tsItems[i]['timeUnit'],
@@ -755,6 +779,7 @@ class FileImport():
                     tsDict['data']['dataPoints'] = valueList[k : k + chunkSize]
                     try:
                         timeseries.TimeSeries.setTimeSeriesDataCollection(self=tsClient, timeSeriesData=[tsDict])
+
                     except Exception as err:
                         logger.error(f"Time series values could not be written. Cause: {err}")
                         errorDict[i+1] = err
