@@ -297,9 +297,11 @@ class FileUtils(UtilsProperty):
         else:
             return str(pytz.timezone('UTC'))
 
-    def _convertTimestamp(timestamp, timeZone, dateTimeFormat):
+    def _convertTimestamp(timestamp, timeZone, dateTimeFormat, timeDelta):
 
         timestamp = datetime.strptime(timestamp, dateTimeFormat)
+        if timeDelta != None:
+            timestamp += timeDelta
         timestamp = pytz.timezone(timeZone).localize(timestamp).isoformat()
         return timestamp
 
