@@ -1,4 +1,5 @@
 from itertools import islice
+from loguru import logger
 
 class UtilsTimeSeries():
 
@@ -9,7 +10,7 @@ class UtilsTimeSeries():
         for item in collection:
             tsData += '{\n'
             for key, value in item.items():
-                if key in ['_inventoryId', '_inventoryItemId']:
+                if key in ['sys_inventoryId', 'sys_inventoryItemId']:
                     tsData += f'  {key}: "{value}"\n'
                 elif key == 'data':
                     tsData += '  data: {\n'
@@ -21,7 +22,10 @@ class UtilsTimeSeries():
                         elif dkey == 'dataPoints':
                             tsData += '    dataPoints: [\n'
                             for dp_key in dvalue:
-                                tsData += f'''     {{timestamp: "{dp_key['timestamp']}", value: {dp_key['value']}}}\n'''
+                                if 'flag' not in dvalue:
+                                    tsData += f'''     {{timestamp: "{dp_key['timestamp']}", value: {dp_key['value']}}}\n'''
+                                else:    
+                                    tsData += f'''     {{timestamp: "{dp_key['timestamp']}", value: {dp_key['value']}, flag: {dp_key['flag']}}}\n'''
                             tsData += '    ]\n'
                         else: pass
                     tsData += '  }\n'
@@ -52,4 +56,5 @@ class UtilsTimeSeries():
     def _sliceDataPoints(dataPoints:dict, start:int, stop:int):
         """Return a slice of the dataPoints dictionary"""
 
-        return dict(islice(dataPoints, start, stop))
\ No newline at end of file
+        return dict(islice(dataPoints, start, stop))
+
